Subject: fix for vulnerability CVE-2010-0406 for OpenTTD 0.6.0 - 1.0.0 (Denial of service (server) via leaking file descriptors)
From: OpenTTD developer team <info@openttd.org>
Origin: backport, https://github.com/OpenTTD/OpenTTD/commit/26af87b
Bug: https://github.com/OpenTTD/OpenTTD/issues/3785

Upon a client downloading the map from the server a file is allocated. If this
download fails for any reason at the client side, e.g. lost connection or the
player cancelling the download, a file descriptor is lost.

Repeating this process enough times can cause OpenTTD to run out of file
descriptors and as a result crash OpenTTD.

The attached patches do not change network compatability at all.


Index: src/network/network_server.cpp
===================================================================
--- src/network/network_server.cpp	(revision 19694)
+++ src/network/network_server.cpp	(revision 19695)
@@ -322,7 +322,7 @@
 	 *    nothing
 	 */

-	static FILE *file_pointer;
+	static FILE *file_pointer = NULL;
 	static uint sent_packets; // How many packets we did send succecfully last time

 	if (cs->status < STATUS_AUTHORIZED) {
@@ -338,6 +338,9 @@
 		if (SaveOrLoad(filename, SL_SAVE, AUTOSAVE_DIR) != SL_OK) usererror("network savedump failed");

+		if (file_pointer != NULL) fclose(file_pointer);
+
 		file_pointer = FioFOpenFile(filename, "rb", AUTOSAVE_DIR);
+		if (file_pointer == NULL) error("network savedump failed - could not open just saved dump");
+
 		fseek(file_pointer, 0, SEEK_END);
-
 		if (ftell(file_pointer) == 0) usererror("network savedump failed - zero sized savegame?");
@@ -382,6 +385,7 @@
 				 *  to send it is ready (maybe that happens like never ;)) */
 				cs->status = STATUS_DONE_MAP;
 				fclose(file_pointer);
+				file_pointer = NULL;

 				NetworkClientSocket *new_cs;
 				bool new_map_client = false;

